@@ -259,7 +259,11 @@ block_exec = function(options) {
     # make sure all objects to be saved exist in env
     objs = intersect(c(objs, obj.new), ls(env, all.names = TRUE))
     if (options$autodep) {
-      cache$objects(objs, code, options$label, options$cache.path)
+      # you shall manually specify global object names if find_symbols() is not reliable
+      cache$objects(
+        objs, options$cache.globals %n% find_symbols(code), options$label,
+        options$cache.path
+      )
       dep_auto()
     }
     if (options$cache < 3) {
@@ -343,7 +347,11 @@ filter_evaluate = function(res, opt, test) {
 
 # find recorded plots in the output of evaluate()
 find_recordedplot = function(x) {
-  vapply(x, evaluate::is.recordedplot, logical(1))
+  vapply(x, is_plot_output, logical(1))
+}
+
+is_plot_output = function(x) {
+  evaluate::is.recordedplot(x) || inherits(x, 'knit_image_paths')
 }
 
 # move plots before source code
@@ -375,7 +383,8 @@ merge_class = function(res, class = c('source', 'message', 'warning')) {
     if (idx2 - idx1 == 1) {
       res2 = res[[idx2]]
       # merge warnings/messages only if next one is identical to previous one
-      if (class == 'source' || identical(res1, res2)) {
+      if (class == 'source' || identical(res1, res2) ||
+          (class == 'message' && !grepl('\n$', tail(res1[[el]], 1)))) {
         res[[k1]][[el]] = c(res[[k1]][[el]], res2[[el]])
         k2 = c(k2, idx2)
       } else {
@@ -444,6 +453,7 @@ process_tangle.block = function(x) {
     try(params[o] <- list(eval_lang(params[[o]])))
   if (isFALSE(params$purl)) return('')
   label = params$label; ev = params$eval
+  if (params$engine != 'R') return(comment_out(knit_code$get(label)))
   code = if (!isFALSE(ev) && !is.null(params$child)) {
     cmds = lapply(sc_split(params$child), knit_child)
     paste(unlist(cmds), collapse = '\n')
@@ -459,13 +469,23 @@ process_tangle.block = function(x) {
 }
 #' @export
 process_tangle.inline = function(x) {
-  if (opts_knit$get('documentation') == 2L) {
-    return(paste(line_prompt(x$input.src, "#' ", "#' "), collapse = '\n'))
-  }
+
+  output = if (opts_knit$get('documentation') == 2L) {
+    output = paste(line_prompt(x$input.src, "#' ", "#' "), collapse = '\n')
+  } else ''
+
   code = x$code
-  if (length(code) == 0L || !any(idx <- grepl('knit_child\\(.+\\)', code)))
-    return('')
-  paste(c(sapply(code[idx], function(z) eval(parse_only(z))), ''), collapse = '\n')
+  if (length(code) == 0L) return(output)
+
+  if (getOption('knitr.purl.inline', FALSE)) output = c(output, code)
+
+  idx = grepl('knit_child\\(.+\\)', code)
+  if (any(idx)) {
+    cout = sapply(code[idx], function(z) eval(parse_only(z)))
+    output = c(output, cout, '')
+  }
+
+  paste(output, collapse = '\n')
 }
 
 
